print("import: 'hamcrest'")
import hamcrest

print("import: 'hamcrest.core'")
import hamcrest.core

print("import: 'hamcrest.core.core'")
import hamcrest.core.core

print("import: 'hamcrest.core.helpers'")
import hamcrest.core.helpers

print("import: 'hamcrest.library'")
import hamcrest.library

print("import: 'hamcrest.library.collection'")
import hamcrest.library.collection

print("import: 'hamcrest.library.integration'")
import hamcrest.library.integration

print("import: 'hamcrest.library.number'")
import hamcrest.library.number

print("import: 'hamcrest.library.object'")
import hamcrest.library.object

print("import: 'hamcrest.library.text'")
import hamcrest.library.text

