from __future__ import absolute_import
"""Matchers that inspect objects and classes."""

from .haslength import has_length
from .hasproperty import has_property, has_properties
from .hasstring import has_string

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"
