from __future__ import absolute_import
"""Matchers that perform numeric comparisons."""

from .iscloseto import close_to
from .ordering_comparison import greater_than, greater_than_or_equal_to, less_than, less_than_or_equal_to

__author__ = "Jon Reid"
__copyright__ = "Copyright 2011 hamcrest.org"
__license__ = "BSD, see License.txt"
